@@ -46,15 +46,20 @@ static void	dmap_mdns_publisher_finalize   (GObject	            *object);
 
 #define DMAP_MDNS_PUBLISHER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_DMAP_MDNS_PUBLISHER, DmapMdnsPublisherPrivate))
 
-struct DmapMdnsPublisherPrivate
+struct DmapMdnsPublisherService
 {
-	AvahiClient     *client;
-	AvahiEntryGroup *entry_group;
-
 	char            *name;
 	guint            port;
 	char		*type_of_service;
 	gboolean         password_required;
+	gchar          **txt_records;
+};
+
+struct DmapMdnsPublisherPrivate
+{
+	AvahiClient     *client;
+	AvahiEntryGroup *entry_group;
+	GSList		*service;
 };
 
 enum {
@@ -83,28 +88,87 @@ dmap_mdns_publisher_error_quark (void)
 	return quark;
 }
 
+static void
+emit_published (char *name, DmapMdnsPublisher *publisher)
+{
+	g_signal_emit (publisher, signals [PUBLISHED], 0, name);
+}
+
 static void
 entry_group_cb (AvahiEntryGroup     *group,
 		AvahiEntryGroupState state,
 		DmapMdnsPublisher *publisher)
 {
 	if (state == AVAHI_ENTRY_GROUP_ESTABLISHED) {
-
-		g_signal_emit (publisher, signals [PUBLISHED], 0, publisher->priv->name);
-
+		g_slist_foreach (publisher->priv->service, (GFunc) emit_published, publisher);
 	} else if (state == AVAHI_ENTRY_GROUP_COLLISION) {
 		g_warning ("MDNS name collision");
 
+		/* FIXME: how to know which name collided?
 		g_signal_emit (publisher, signals [NAME_COLLISION], 0, publisher->priv->name);
+		*/
+		g_signal_emit (publisher, signals [NAME_COLLISION], 0, "unknown");
 	}
 }
 
 static gboolean
-create_service (DmapMdnsPublisher *publisher,
-		GError             **error)
+create_service (struct DmapMdnsPublisherService *service,
+		DmapMdnsPublisher *publisher,
+		GError **error)
 {
+	int ret;
+	const char *password_record;
+	AvahiStringList *txt_records;
+
+	if (service->password_required) {
+		password_record = "Password=true";
+	} else {
+		password_record = "Password=false";
+	}
+	
+	txt_records = avahi_string_list_new(password_record, NULL);
+	
+	if (service->txt_records) {
+		//g_debug("Number of txt records: %d", service->txt_records);
+		gchar **txt_record = service->txt_records;
+		while (*txt_record) {
+			txt_records = avahi_string_list_add(txt_records, *txt_record);
+			txt_record++;
+		}
+	}
+
+	ret = avahi_entry_group_add_service_strlst (publisher->priv->entry_group,
+					     AVAHI_IF_UNSPEC,
+					     AVAHI_PROTO_UNSPEC,
+					     0,
+					     service->name,
+					     service->type_of_service,
+					     NULL,
+					     NULL,
+					     service->port,
+					     txt_records);
+					     
+	avahi_string_list_free(txt_records);
+
+	if (ret < 0) {
+		g_set_error (error,
+			     DMAP_MDNS_PUBLISHER_ERROR,
+			     DMAP_MDNS_PUBLISHER_ERROR_FAILED,
+			     "%s: %s",
+			     _("Could not add service"),
+			     avahi_strerror (ret));
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static gboolean
+create_services (DmapMdnsPublisher *publisher,
+		 GError             **error)
+{
+	GSList     *ptr;
 	int         ret;
-	const char *txt_record;
 
 	if (publisher->priv->entry_group == NULL) {
 		publisher->priv->entry_group = avahi_entry_group_new (publisher->priv->client,
@@ -125,39 +189,10 @@ create_service (DmapMdnsPublisher *publisher,
 		return FALSE;
 	}
 
-#if 0
-	g_message ("Service name:%s port:%u password:%d",
-		   publisher->priv->name,
-		   publisher->priv->port,
-		   publisher->priv->password_required);
-#endif
-
-	if (publisher->priv->password_required) {
-		txt_record = "Password=true";
-	} else {
-		txt_record = "Password=false";
-	}
-
-	ret = avahi_entry_group_add_service (publisher->priv->entry_group,
-					     AVAHI_IF_UNSPEC,
-					     AVAHI_PROTO_UNSPEC,
-					     0,
-					     publisher->priv->name,
-					     publisher->priv->type_of_service,
-					     NULL,
-					     NULL,
-					     publisher->priv->port,
-					     txt_record,
-					     NULL);
-
-	if (ret < 0) {
-		g_set_error (error,
-			     DMAP_MDNS_PUBLISHER_ERROR,
-			     DMAP_MDNS_PUBLISHER_ERROR_FAILED,
-			     "%s: %s",
-			     _("Could not add service"),
-			     avahi_strerror (ret));
-		return FALSE;
+	for (ptr = publisher->priv->service; ptr; ptr = g_slist_next (ptr)) {
+		if (! create_service (ptr->data, publisher, error)) {
+			return FALSE;
+		}
 	}
 
 	ret = avahi_entry_group_commit (publisher->priv->entry_group);
@@ -176,112 +211,51 @@ create_service (DmapMdnsPublisher *publisher,
 }
 
 static gboolean
-refresh_service (DmapMdnsPublisher *publisher,
-		 GError             **error)
+refresh_services (DmapMdnsPublisher *publisher,
+		  GError             **error)
 {
-	return create_service (publisher, error);
+	return create_services (publisher, error);
 }
 
-static gboolean
-publisher_set_name_internal (DmapMdnsPublisher *publisher,
-			     const char          *name,
-			     GError             **error)
+static struct DmapMdnsPublisherService *
+find_service_by_port (GSList *list, guint port)
 {
-	g_free (publisher->priv->name);
-	publisher->priv->name = g_strdup (name);
+	GSList *ptr;
 
-	return TRUE;
-}
-
-gboolean
-dmap_mdns_publisher_set_name (DmapMdnsPublisher *publisher,
-				 const char          *name,
-				 GError             **error)
-{
-        g_return_val_if_fail (publisher != NULL, FALSE);
-
-	publisher_set_name_internal (publisher, name, error);
-
-	if (publisher->priv->entry_group) {
-		refresh_service (publisher, error);
+	for (ptr = list; ptr; ptr = g_slist_next (ptr)) {
+		if (port == ((struct DmapMdnsPublisherService *) ptr->data)->port)
+			break;
 	}
 
-	return TRUE;
-}
-
-static gboolean
-publisher_set_port_internal (DmapMdnsPublisher *publisher,
-			     guint                port,
-			     GError             **error)
-{
-	publisher->priv->port = port;
-
-	return TRUE;
+	return ptr ? ptr->data : NULL;
 }
 
 gboolean
-dmap_mdns_publisher_set_port (DmapMdnsPublisher *publisher,
-				 guint                port,
-				 GError             **error)
+dmap_mdns_publisher_rename_at_port (DmapMdnsPublisher *publisher,
+				    guint	       port,
+				    const char        *name,
+				    GError           **error)
 {
+	struct DmapMdnsPublisherService *ptr;
+
         g_return_val_if_fail (publisher != NULL, FALSE);
 
-	publisher_set_port_internal (publisher, port, error);
+	ptr = find_service_by_port (publisher->priv->service, port);
 
-	if (publisher->priv->entry_group) {
-		refresh_service (publisher, error);
+	if (ptr == NULL) {
+                g_set_error (error,
+			     DMAP_MDNS_PUBLISHER_ERROR,
+			     DMAP_MDNS_PUBLISHER_ERROR_FAILED,
+                             "%s",
+                             _("No service at port"));
+		return FALSE;
 	}
 
-	return TRUE;
-}
-
-static gboolean
-publisher_set_type_of_service_internal (DmapMdnsPublisher *publisher,
-			     const char          *type_of_service,
-			     GError             **error)
-{
-	g_free (publisher->priv->type_of_service);
-	publisher->priv->type_of_service = g_strdup (type_of_service);
-
-	return TRUE;
-}
-
-gboolean
-dmap_mdns_publisher_set_type_of_service (DmapMdnsPublisher *publisher,
-				 const char          *type_of_service,
-				 GError             **error)
-{
-        g_return_val_if_fail (publisher != NULL, FALSE);
-
-	publisher_set_type_of_service_internal (publisher, type_of_service, error);
+	g_free (ptr->name);
+	ptr->name = g_strdup (name);
 
 	if (publisher->priv->entry_group) {
-		refresh_service (publisher, error);
-	}
-
-	return TRUE;
-}
-
-static gboolean
-publisher_set_password_required_internal (DmapMdnsPublisher *publisher,
-					  gboolean             required,
-					  GError             **error)
-{
-	publisher->priv->password_required = required;
-	return TRUE;
-}
-
-gboolean
-dmap_mdns_publisher_set_password_required (DmapMdnsPublisher *publisher,
-					      gboolean             required,
-					      GError             **error)
-{
-        g_return_val_if_fail (publisher != NULL, FALSE);
-
-	publisher_set_password_required_internal (publisher, required, error);
-
-	if (publisher->priv->entry_group) {
-		refresh_service (publisher, error);
+		refresh_services (publisher, error);
 	}
 
 	return TRUE;
@@ -293,8 +267,11 @@ dmap_mdns_publisher_publish (DmapMdnsPublisher *publisher,
 				guint                port,
 				const char          *type_of_service,
 				gboolean             password_required,
+				gchar              **txt_records,
 				GError             **error)
 {
+	struct DmapMdnsPublisherService *service;
+
 	if (publisher->priv->client == NULL) {
                 g_set_error (error,
 			     DMAP_MDNS_PUBLISHER_ERROR,
@@ -304,18 +281,35 @@ dmap_mdns_publisher_publish (DmapMdnsPublisher *publisher,
 		return FALSE;
 	}
 
-	publisher_set_name_internal (publisher, name, NULL);
-	publisher_set_port_internal (publisher, port, NULL);
-	publisher_set_type_of_service_internal (publisher, type_of_service, NULL);
-	publisher_set_password_required_internal (publisher, password_required, NULL);
+	
+	service = g_new (struct DmapMdnsPublisherService, 1);
 
-	return create_service (publisher, error);
+	service->name = g_strdup(name);
+	service->port = port;
+	service->type_of_service = g_strdup (type_of_service);
+	service->password_required = password_required;
+	service->txt_records = g_strdupv (txt_records);
+
+	publisher->priv->service = g_slist_append (publisher->priv->service, service);
+
+	return create_services (publisher, error);
+}
+
+static void
+free_service (struct DmapMdnsPublisherService *service, gpointer user_data)
+{
+	g_free (service->name);
+	g_free (service->type_of_service);
+	g_strfreev (service->txt_records);
 }
 
 gboolean
 dmap_mdns_publisher_withdraw (DmapMdnsPublisher *publisher,
-				 GError             **error)
+			      guint port,
+			      GError             **error)
 {
+	struct DmapMdnsPublisherService *ptr;
+
 	if (publisher->priv->client == NULL) {
                 g_set_error (error,
 			     DMAP_MDNS_PUBLISHER_ERROR,
@@ -325,7 +319,8 @@ dmap_mdns_publisher_withdraw (DmapMdnsPublisher *publisher,
 		return FALSE;
 	}
 
-	if (publisher->priv->entry_group == NULL) {
+	if (publisher->priv->entry_group == NULL
+	    || ! (ptr = find_service_by_port (publisher->priv->service, port))) {
                 g_set_error (error,
 			     DMAP_MDNS_PUBLISHER_ERROR,
 			     DMAP_MDNS_PUBLISHER_ERROR_FAILED,
@@ -334,9 +329,18 @@ dmap_mdns_publisher_withdraw (DmapMdnsPublisher *publisher,
 		return FALSE;
 	}
 
-	avahi_entry_group_reset (publisher->priv->entry_group);
-	avahi_entry_group_free (publisher->priv->entry_group);
-	publisher->priv->entry_group = NULL;
+	free_service (ptr, NULL);
+	publisher->priv->service = g_slist_remove (publisher->priv->service, ptr);
+
+	if (publisher->priv->service == NULL) {
+		avahi_entry_group_reset (publisher->priv->entry_group);
+		avahi_entry_group_free (publisher->priv->entry_group);
+		publisher->priv->entry_group = NULL;
+	} else {
+		create_services (publisher, error);
+		if (error != NULL)
+			return FALSE;
+	}
 
 	return TRUE;
 }
@@ -367,11 +371,30 @@ dmap_mdns_publisher_get_property (GObject	 *object,
 	}
 }
 
+static GObject *
+dmap_mdns_publisher_constructor (GType type,
+				 guint n_construct_params,
+				 GObjectConstructParam *construct_params)
+{
+	/* This class is a singleton. */
+	static GObject *self = NULL;
+
+	if (self == NULL) {
+		self = G_OBJECT_CLASS (dmap_mdns_publisher_parent_class)->constructor (
+			type, n_construct_params, construct_params);
+		g_object_add_weak_pointer (self, (gpointer) &self);
+		return self;
+	}
+
+	return g_object_ref (self);
+}
+
 static void
 dmap_mdns_publisher_class_init (DmapMdnsPublisherClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+	object_class->constructor  = dmap_mdns_publisher_constructor;
 	object_class->finalize	   = dmap_mdns_publisher_finalize;
 	object_class->get_property = dmap_mdns_publisher_get_property;
 	object_class->set_property = dmap_mdns_publisher_set_property;
@@ -405,7 +428,9 @@ dmap_mdns_publisher_init (DmapMdnsPublisher *publisher)
 {
 	publisher->priv = DMAP_MDNS_PUBLISHER_GET_PRIVATE (publisher);
 
-	publisher->priv->client = dmap_mdns_avahi_get_client ();
+	publisher->priv->client      = dmap_mdns_avahi_get_client ();
+	publisher->priv->entry_group = NULL;
+	publisher->priv->service     = NULL;
 }
 
 static void
@@ -425,10 +450,20 @@ dmap_mdns_publisher_finalize (GObject *object)
 		publisher->priv->entry_group = NULL;
 	}
 
-	avahi_client_free (publisher->priv->client);
+	/* FIXME: dmap_mdns_avahi_get_client() ensures that a client is initialized only
+	 * once during the lifetime of a program. This needs to be changed so that the
+	 * following call works. Otherwise, an application like Rhythmbox will crash if 
+	 * a user deactivates and then activates the DAAP plugin. In this case,
+	 * publisher->priv->client will be free'd but will not be allocated in a
+	 * successive call to dmap_mdns_avahi_get_client().
+	 *
+	 * avahi_client_free (publisher->priv->client);
+	 */
 
-	g_free (publisher->priv->name);
-	g_free (publisher->priv->type_of_service);
+	g_slist_foreach (publisher->priv->service, (GFunc) free_service, NULL);
+	g_slist_free (publisher->priv->service);
+
+	publisher_object = NULL;
 
 	G_OBJECT_CLASS (dmap_mdns_publisher_parent_class)->finalize (object);
 }
