@@ -78,7 +78,7 @@ static void databases_items_xxx (DMAPShare *share,
 				 GHashTable *query,
 				 SoupClientContext *context);
 static struct DMAPMetaDataMap *get_meta_data_map (DMAPShare *share);
-static guint32 add_entry_to_mlcl (gpointer id,
+static void add_entry_to_mlcl (gpointer id,
 			       DMAPRecord *record,
 			       gpointer mb);
 
@@ -330,6 +330,10 @@ file_to_mmap (const char *location)
         GError *error = NULL;
 
         file = g_file_new_for_uri (location);
+	/* NOTE: this is broken if original filename contains "%20" etc. This
+	 * is because g_file_get_path() will translate this to " ", etc. But
+	 * the filename really may have used "%20" (not " ").
+	 */
         path = g_file_get_path (file);
         if (path == NULL) {
                 g_warning ("Couldn't mmap %s: couldn't get path", path);
@@ -347,7 +351,7 @@ file_to_mmap (const char *location)
 	return mapped_file;
 }
 
-static guint32
+static void
 add_entry_to_mlcl (gpointer id,
 		   DMAPRecord *record,
 		   gpointer _mb)
@@ -457,7 +461,9 @@ add_entry_to_mlcl (gpointer id,
 
 			mapped_file = file_to_mmap (location);
 			if (mapped_file == NULL) {
-				g_error ("Error opening %s", location);
+				g_warning ("Error opening %s", location);
+				data = NULL;
+				size = 0;
 			} else {
 				data = (unsigned char *) g_mapped_file_get_contents (mapped_file);
 				size = g_mapped_file_get_length (mapped_file);
@@ -466,8 +472,6 @@ add_entry_to_mlcl (gpointer id,
 		}
 		dmap_structure_add (mlit, DMAP_CC_PFDT, data, size);
 	}
-
-	return ((DMAPStructureItem *) mlit->data)->size;
 }
 
 static void
